import { AxiosInstance } from 'axios';

import { ActivatedFeaturesApiFactory } from './Coscine.Api.ActivatedFeatures/api';
import { Configuration } from './Coscine.Api.ActivatedFeatures/configuration';
import { AdminApiFactory } from './Coscine.Api.Admin/api';
import { BlobApiFactory } from './Coscine.Api.Blob/api';
import { MetadataApiFactory } from './Coscine.Api.Metadata/api';
import { NoticeApiFactory } from './Coscine.Api.Notices/api';
import { OrganizationApiFactory } from './Coscine.Api.Organization/api';
import { DisciplineApiFactory } from './Coscine.Api.Project/api';
import { LicenseApiFactory } from './Coscine.Api.Project/api';
import { ProjectApiFactory } from './Coscine.Api.Project/api';
import { ProjectRoleApiFactory } from './Coscine.Api.Project/api';
import { RoleApiFactory } from './Coscine.Api.Project/api';
import { SearchApiFactory } from './Coscine.Api.Project/api';
import { SubProjectApiFactory } from './Coscine.Api.Project/api';
import { VisibilityApiFactory } from './Coscine.Api.Project/api';
import { QuotaApiFactory } from './Coscine.Api.Quota/api';
import { ResourceApiFactory } from './Coscine.Api.Resources/api';
import { ResourceTypeApiFactory } from './Coscine.Api.Resources/api';
import { SemanticSearchApiFactory } from './Coscine.Api.Search/api';
import { TokenApiFactory } from './Coscine.Api.Token/api';
import { TreeApiFactory } from './Coscine.Api.Tree/api';
import { ContactChangeApiFactory } from './Coscine.Api.User/api';
import { LanguageApiFactory } from './Coscine.Api.User/api';
import { TOSApiFactory } from './Coscine.Api.User/api';
import { TitleApiFactory } from './Coscine.Api.User/api';
import { UserApiFactory } from './Coscine.Api.User/api';

let accessToken = '';
if (typeof coscine !== 'undefined' && typeof coscine.authorization !== 'undefined') {
  accessToken = 'Bearer ' + coscine.authorization.bearer;
}
if (typeof window !== 'undefined') {
  // LocalStorage > Global Variables
  const localStorageToken = localStorage.getItem('coscine.authorization.bearer');
  if (localStorageToken) {
    accessToken = 'Bearer ' + localStorageToken;
  }
}

const getHostName = () => {
  let hostName = typeof window !== 'undefined' ? window.location.hostname : 'coscine.rwth-aachen.de';
  if (hostName.indexOf(':') !== -1) {
    if (hostName.indexOf('https://') !== -1) {
      hostName = hostName.replace('https://', '');
    }
    hostName = hostName.substr(0, hostName.indexOf(':'));
  }
  return hostName;
};

function implementations(axios?: AxiosInstance) {
  return {
    ActivatedFeaturesApi: ActivatedFeaturesApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.ActivatedFeatures', axios),
    AdminApi: AdminApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Admin', axios),
    BlobApi: BlobApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Blob', axios),
    ContactChangeApi: ContactChangeApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.User', axios),
    DisciplineApi: DisciplineApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Project', axios),
    LanguageApi: LanguageApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.User', axios),
    LicenseApi: LicenseApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Project', axios),
    MetadataApi: MetadataApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Metadata', axios),
    NoticeApi: NoticeApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Notices', axios),
    OrganizationApi: OrganizationApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Organization', axios),
    ProjectApi: ProjectApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Project', axios),
    ProjectRoleApi: ProjectRoleApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Project', axios),
    QuotaApi: QuotaApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Quota', axios),
    ResourceApi: ResourceApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Resources', axios),
    ResourceTypeApi: ResourceTypeApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Resources', axios),
    RoleApi: RoleApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Project', axios),
    SearchApi: SearchApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Project', axios),
    SemanticSearchApi: SemanticSearchApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Search', axios),
    SubProjectApi: SubProjectApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Project', axios),
    TitleApi: TitleApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.User', axios),
    TokenApi: TokenApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Token', axios),
    TOSApi: TOSApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.User', axios),
    TreeApi: TreeApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Tree', axios),
    UserApi: UserApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.User', axios),
    VisibilityApi: VisibilityApiFactory(new Configuration({ 'accessToken': accessToken }), 'https://' + getHostName() + '/coscine/api/Coscine.Api.Project', axios)
  };
};

export default implementations;
