import axios, { AxiosRequestConfig } from 'axios';

const instance = axios.create();

const authHeaderKey = 'Authorization';
const clientCorrolationIdKey = 'X-Coscine-Logging-CorrelationId';

function setHeader(request: AxiosRequestConfig) {
  if (typeof coscine !== "undefined") {
    if (typeof coscine.authorization !== "undefined") {
      request.headers.common[authHeaderKey] =
        'Bearer ' + coscine.authorization.bearer;
    }
    if (typeof coscine.clientcorrolation !== "undefined") {
      request.headers.common[clientCorrolationIdKey] = coscine.clientcorrolation.id;
    }
  }
  if (typeof window !== 'undefined') {
    // LocalStorage > Global Variables
    const localStorageToken = localStorage.getItem("coscine.authorization.bearer");
    if (localStorageToken) {
      request.headers.common[authHeaderKey] =
          'Bearer ' + localStorageToken;
    }
    const localStorageClientCorrelation = localStorage.getItem("coscine.clientcorrolation.id");
    if (localStorageClientCorrelation) {
      request.headers.common[clientCorrolationIdKey] = localStorageClientCorrelation;
    }
  }
};

function increaseLoading() {
  if ((typeof coscine !== "undefined") && (typeof coscine.loading !== "undefined") && (typeof coscine.loading.counter !== "undefined")) {
    coscine.loading.counter++;
  }
  if (typeof window !== 'undefined') {
    const localStorageLoading = localStorage.getItem("coscine.loading.counter");
    if (localStorageLoading) {
      localStorage.setItem("coscine.loading.counter", String(Number(localStorageLoading) + 1));
    }
  }
}

function decreaseLoading() {
  if ((typeof coscine !== "undefined") && (typeof coscine.loading !== "undefined") && (typeof coscine.loading.counter !== "undefined")) {
    coscine.loading.counter--;
  }
  if (typeof window !== 'undefined') {
    const localStorageLoading = localStorage.getItem("coscine.loading.counter");
    if (localStorageLoading) {
      localStorage.setItem("coscine.loading.counter", String(Number(localStorageLoading) - 1));
    }
  }
}

instance.interceptors.request.use((request: AxiosRequestConfig) => {
  setHeader(request);
  increaseLoading();
  return request;
}, (error: unknown) => {
  return Promise.reject(error);
});

instance.interceptors.response.use((response: AxiosRequestConfig) => {
  decreaseLoading();
  return response;
}, (error: unknown) => {
  decreaseLoading();
  return Promise.reject(error);
});

export default instance;
